@@ -1,5 +1,5 @@
 /*
- * sadf_misc.c: Funtions used by sadf to display special records
+ * sadf_misc.c: Functions used by sadf to display special records
  * (C) 2011-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
@@ -1276,7 +1276,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 				printf("x%d", fal->nr2);
 			}
 			printf("\t(%d,%d,%d)", fal->types_nr[0], fal->types_nr[1], fal->types_nr[2]);
-			if ((p >= 0) && (act[p]->magic == ACTIVITY_MAGIC_UNKNOWN)) {
+			if ((p >= 0) && (act[p]->magic != fal->magic)) {
 				printf(_(" \t[Unknown format]"));
 			}
 			printf("\n");
